@@ -2,12 +2,12 @@
  * Internal support functions for the remctld daemon.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
+ * Copyright 2015-2016, 2018 Russ Allbery <eagle@eyrie.org>
  * Copyright 2016 Dropbox, Inc.
- * Copyright 2015, 2016 Russ Allbery <eagle@eyrie.org>
- * Copyright 2006, 2007, 2008, 2009, 2010, 2012, 2014
+ * Copyright 2006-2010, 2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
- * See LICENSE for licensing terms.
+ * SPDX-License-Identifier: MIT
  */
 
 #ifndef SERVER_INTERNAL_H
@@ -81,7 +81,7 @@ struct client {
 /* Holds the configuration for a single command. */
 struct rule {
     char *file;                 /* Config file name. */
-    int lineno;                 /* Config file line number. */
+    size_t lineno;              /* Config file line number. */
     struct vector *line;        /* The split configuration line. */
     char *command;              /* Command (first argument). */
     char *subcommand;           /* Subcommand (second argument). */
@@ -190,7 +190,8 @@ struct iovec **server_ssh_parse_command(const char *);
 
 /* libevent utility functions. */
 void server_event_log_callback(int, const char *);
-void server_event_fatal_callback(int);
+void server_event_fatal_callback(int)
+    __attribute__((__noreturn__));
 
 END_DECLS
 
